#!/bin/sh

# Docker entrypoint

cd `dirname "$0"`

. ./config.sh

# Creating directory and set permissions
mkdir -p ${BASEDIR}/logs ${BASEDIR}/onyx/log
chown -R www-data:www-data ${BASEDIR}/pki ${BASEDIR}/PKI ${BASEDIR}/onyx/log
chown -R ${SYNCHRO_USER}:www-data ${BASEDIR}/submission ${BASEDIR}/logs ${BASEDIR}/out
chmod 770 ${BASEDIR}/submission
chown ${SYNCHRO_USER} ${BASEDIR}/.ssh/id_rsa
chown www-data:www-data ${BASEDIR}/shared ${BASEDIR}/shared/crl.pem

# Update database profile
cat <<EOF > ${BASEDIR}/onyx/db/docker.profile.php &&
<?php

if(!defined('ONYX')) exit;

\$___profile['db'] = '$DB_ENV_MYSQL_DATABASE';
\$___profile['host'] = '$DB_PORT_3306_TCP_ADDR';
\$___profile['user'] = '$DB_ENV_MYSQL_USER';
\$___profile['pass'] = '$DB_ENV_MYSQL_PASSWORD';
EOF
sed -i 's/"profile">sample</"profile">docker</' ${BASEDIR}/onyx/config/root.xml

# Development version?
if [ -n "$DEVELOPMENT" ]
then
    echo -e "\033[1;31mDEVELOPMENT version ENABLED\033[0m"
    sed -i 's/<var name="development">0</<var name="development">1</' ${BASEDIR}/onyx/config/root.xml
    if [ "$DEVELOPMENT" -gt 0 ]
    then
	sed -i 's/<env option="\(display.*_errors\)">0</<env option="\1">1</' ${BASEDIR}/onyx/config/root.xml
    fi
else
    echo -e "\033[32mLaunched as PRODUCTION version\033[0m"
fi

# Generate hashed path for files
echo -n 'Copying files... '
${BASEDIR}/gen_hash_link_files.sh --copy ${BASEDIR}/files-in ${BASEDIR}/files
echo -e "\033[32mdone\033[0m"

echo -e "\033[1;33mYou should run in background:\033[0m tail -f ./logs/checks.log &"

# Continue execution
exec "$@"
